From 627b1b704381acb80e46edd218f7968939adf84e Mon Sep 17 00:00:00 2001
From: Wu Jianhua <jianhua.wu@intel.com>
Date: Wed, 23 Feb 2022 20:18:47 +0800
Subject: [PATCH 08/10] avfilter/vf_blend_vulkan: add lighten blend mode

Use the commands below to test: (href: https://trac.ffmpeg.org/wiki/Blend)

I. make an image for test
ffmpeg -f lavfi -i color=s=256x256,geq=r='H-1-Y':g='H-1-Y':b='H-1-Y' -frames 1 \
-y -pix_fmt yuv420p test.jpg

II. blend in sw
ffmpeg -i test.jpg -vf "split[a][b];[b]transpose[b];[a][b]blend=all_mode=lighten,\
pseudocolor=preset=turbo" -y lighten_sw.jpg

III. blend in vulkan
ffmpeg -init_hw_device vulkan -i test.jpg -vf "split[a][b];[b]transpose[b];\
[a]hwupload[a];[b]hwupload[b];[a][b]blend_vulkan=all_mode=lighten,hwdownload,\
format=yuv420p,pseudocolor=preset=turbo" -y lighten_vulkan.jpg

Signed-off-by: Wu Jianhua <jianhua.wu@intel.com>
---
 libavfilter/vf_blend_vulkan.c | 3 +++
 1 file changed, 3 insertions(+)

diff --git a/libavfilter/vf_blend_vulkan.c b/libavfilter/vf_blend_vulkan.c
index b5237ad36b..558cef7064 100644
--- a/libavfilter/vf_blend_vulkan.c
+++ b/libavfilter/vf_blend_vulkan.c
@@ -90,6 +90,7 @@ DEFINE_BLEND_MODE(NEGATION,   FN(MAX - abs(MAX - A - B)))
 DEFINE_BLEND_MODE(EXTREMITY,  FN(abs(MAX - A - B)))
 DEFINE_BLEND_MODE(DIFFERENCE, FN(abs(A - B)))
 DEFINE_BLEND_MODE(DARKEN,     FN(min(A, B)))
+DEFINE_BLEND_MODE(LIGHTEN,    FN(max(A, B)))
 
 static inline void init_blend_func(FilterParamsVulkan *param)
 {
@@ -108,6 +109,7 @@ static inline void init_blend_func(FilterParamsVulkan *param)
     CASE(EXTREMITY)
     CASE(DIFFERENCE)
     CASE(DARKEN)
+    CASE(LIGHTEN)
     default: param->blend = NULL; break;
     }
 
@@ -486,6 +488,7 @@ static const AVOption blend_vulkan_options[] = {
         { "extremity",  "", 0, AV_OPT_TYPE_CONST, { .i64 = BLEND_EXTREMITY  }, 0, 0, FLAGS, "mode" },
         { "difference", "", 0, AV_OPT_TYPE_CONST, { .i64 = BLEND_DIFFERENCE }, 0, 0, FLAGS, "mode" },
         { "darken",     "", 0, AV_OPT_TYPE_CONST, { .i64 = BLEND_DARKEN     }, 0, 0, FLAGS, "mode" },
+        { "lighten",    "", 0, AV_OPT_TYPE_CONST, { .i64 = BLEND_LIGHTEN    }, 0, 0, FLAGS, "mode" },
 
     { "c0_opacity",  "set color component #0 opacity", OFFSET(params[0].opacity), AV_OPT_TYPE_DOUBLE, { .dbl = 1 }, 0, 1, FLAGS },
     { "c1_opacity",  "set color component #1 opacity", OFFSET(params[1].opacity), AV_OPT_TYPE_DOUBLE, { .dbl = 1 }, 0, 1, FLAGS },
-- 
2.25.1

